//
// Alignment.h
//
// Library: Foundation
// Package: Dynamic
// Module:  Alignment
//
// Definition of the Alignment class.
//
// Copyright (c) 2007, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// SPDX-License-Identifier:	BSL-1.0
//


#ifndef Foundation_Alignment_INCLUDED
#define Foundation_Alignment_INCLUDED


#include <type_traits>
#define POCO_HAVE_ALIGNMENT


#endif // Foundation_Alignment_INCLUDED
